@@ -13,48 +13,49 @@ class GamepadData(object):
 
 
 class AxisMapping(object):
-    '''Represents a mapping between a HID type
-    and an evdev event'''
+    """Represents a mapping between a HID type
+    and an evdev event"""
+
     def __init__(self, hid, evdev=None):
         self.hid = hid.lower()
 
         if evdev is None:
-            evdev = f'ABS_{hid.upper()}'
+            evdev = f"ABS_{hid.upper()}"
 
-        self.evdev = libevdev.evbit('EV_ABS', evdev)
+        self.evdev = libevdev.evbit("EV_ABS", evdev)
 
 
 class BaseGamepad(BaseDevice):
     buttons_map = {
-        1: 'BTN_SOUTH',
-        2: 'BTN_EAST',
-        3: 'BTN_C',
-        4: 'BTN_NORTH',
-        5: 'BTN_WEST',
-        6: 'BTN_Z',
-        7: 'BTN_TL',
-        8: 'BTN_TR',
-        9: 'BTN_TL2',
-        10: 'BTN_TR2',
-        11: 'BTN_SELECT',
-        12: 'BTN_START',
-        13: 'BTN_MODE',
-        14: 'BTN_THUMBL',
-        15: 'BTN_THUMBR',
+        1: "BTN_SOUTH",
+        2: "BTN_EAST",
+        3: "BTN_C",
+        4: "BTN_NORTH",
+        5: "BTN_WEST",
+        6: "BTN_Z",
+        7: "BTN_TL",
+        8: "BTN_TR",
+        9: "BTN_TL2",
+        10: "BTN_TR2",
+        11: "BTN_SELECT",
+        12: "BTN_START",
+        13: "BTN_MODE",
+        14: "BTN_THUMBL",
+        15: "BTN_THUMBR",
     }
 
     axes_map = {
-        'left_stick': {
-            'x': AxisMapping('x'),
-            'y': AxisMapping('y'),
+        "left_stick": {
+            "x": AxisMapping("x"),
+            "y": AxisMapping("y"),
         },
-        'right_stick': {
-            'x': AxisMapping('z'),
-            'y': AxisMapping('Rz'),
+        "right_stick": {
+            "x": AxisMapping("z"),
+            "y": AxisMapping("Rz"),
         },
     }
 
-    def __init__(self, rdesc, application='Game Pad', name=None, input_info=None):
+    def __init__(self, rdesc, application="Game Pad", name=None, input_info=None):
         assert rdesc is not None
         super().__init__(name, application, input_info=input_info, rdesc=rdesc)
         self.buttons = (1, 2, 3)
@@ -62,6 +63,7 @@ class BaseGamepad(BaseDevice):
         self.left = (127, 127)
         self.right = (127, 127)
         self.hat_switch = 15
+        assert self.parsed_rdesc is not None
 
         self.fields = []
         for r in self.parsed_rdesc.input_reports.values():
@@ -71,10 +73,19 @@ class BaseGamepad(BaseDevice):
     def store_axes(self, which, gamepad, data):
         amap = self.axes_map[which]
         x, y = data
-        setattr(gamepad, amap['x'].hid, x)
-        setattr(gamepad, amap['y'].hid, y)
+        setattr(gamepad, amap["x"].hid, x)
+        setattr(gamepad, amap["y"].hid, y)
 
-    def create_report(self, *, left=(None, None), right=(None, None), hat_switch=None, buttons=None, reportID=None, application='Game Pad'):
+    def create_report(
+        self,
+        *,
+        left=(None, None),
+        right=(None, None),
+        hat_switch=None,
+        buttons=None,
+        reportID=None,
+        application="Game Pad",
+    ):
         """
         Return an input report for this device.
 
@@ -93,7 +104,9 @@ class BaseGamepad(BaseDevice):
         if buttons is not None:
             for i, b in buttons.items():
                 if i not in self.buttons:
-                    raise InvalidHIDCommunication(f'button {i} is not part of this {self.application}')
+                    raise InvalidHIDCommunication(
+                        f"button {i} is not part of this {self.application}"
+                    )
                 if b is not None:
                     self._buttons[i] = b
 
@@ -123,14 +136,18 @@ class BaseGamepad(BaseDevice):
 
         gamepad = GamepadData()
         for i, b in self._buttons.items():
-            gamepad.__setattr__(f'b{i}', int(b) if b is not None else 0)
+            gamepad.__setattr__(f"b{i}", int(b) if b is not None else 0)
 
-        self.store_axes('left_stick', gamepad, left)
-        self.store_axes('right_stick', gamepad, right)
-        gamepad.hatswitch = hat_switch
-        return super().create_report(gamepad, reportID=reportID, application=application)
+        self.store_axes("left_stick", gamepad, left)
+        self.store_axes("right_stick", gamepad, right)
+        gamepad.hatswitch = hat_switch  # type: ignore  ### gamepad is by default empty
+        return super().create_report(
+            gamepad, reportID=reportID, application=application
+        )
 
-    def event(self, *, left=(None, None), right=(None, None), hat_switch=None, buttons=None):
+    def event(
+        self, *, left=(None, None), right=(None, None), hat_switch=None, buttons=None
+    ):
         """
         Send an input event on the default report ID.
 
@@ -143,43 +160,54 @@ class BaseGamepad(BaseDevice):
         :param buttons: a dict of index/bool for the button states,
             where ``None`` is "leave unchanged"
         """
-        r = self.create_report(left=left, right=right, hat_switch=hat_switch, buttons=buttons)
+        r = self.create_report(
+            left=left, right=right, hat_switch=hat_switch, buttons=buttons
+        )
         self.call_input_event(r)
         return [r]
 
 
 class JoystickGamepad(BaseGamepad):
     buttons_map = {
-        1: 'BTN_TRIGGER',
-        2: 'BTN_THUMB',
-        3: 'BTN_THUMB2',
-        4: 'BTN_TOP',
-        5: 'BTN_TOP2',
-        6: 'BTN_PINKIE',
-        7: 'BTN_BASE',
-        8: 'BTN_BASE2',
-        9: 'BTN_BASE3',
-        10: 'BTN_BASE4',
-        11: 'BTN_BASE5',
-        12: 'BTN_BASE6',
-        13: 'BTN_DEAD',
+        1: "BTN_TRIGGER",
+        2: "BTN_THUMB",
+        3: "BTN_THUMB2",
+        4: "BTN_TOP",
+        5: "BTN_TOP2",
+        6: "BTN_PINKIE",
+        7: "BTN_BASE",
+        8: "BTN_BASE2",
+        9: "BTN_BASE3",
+        10: "BTN_BASE4",
+        11: "BTN_BASE5",
+        12: "BTN_BASE6",
+        13: "BTN_DEAD",
     }
 
     axes_map = {
-        'left_stick': {
-            'x': AxisMapping('x'),
-            'y': AxisMapping('y'),
+        "left_stick": {
+            "x": AxisMapping("x"),
+            "y": AxisMapping("y"),
         },
-        'right_stick': {
-            'x': AxisMapping('rudder'),
-            'y': AxisMapping('throttle'),
+        "right_stick": {
+            "x": AxisMapping("rudder"),
+            "y": AxisMapping("throttle"),
         },
     }
 
-    def __init__(self, rdesc, application='Joystick', name=None, input_info=None):
+    def __init__(self, rdesc, application="Joystick", name=None, input_info=None):
         super().__init__(rdesc, application, name, input_info)
 
-    def create_report(self, *, left=(None, None), right=(None, None), hat_switch=None, buttons=None, reportID=None, application=None):
+    def create_report(
+        self,
+        *,
+        left=(None, None),
+        right=(None, None),
+        hat_switch=None,
+        buttons=None,
+        reportID=None,
+        application=None,
+    ):
         """
         Return an input report for this device.
 
@@ -195,14 +223,22 @@ class JoystickGamepad(BaseGamepad):
         :param application: the application for this report, if needed
         """
         if application is None:
-            application = 'Joystick'
-        return super().create_report(left=left, right=right, hat_switch=hat_switch, buttons=buttons, reportID=reportID, application=application)
+            application = "Joystick"
+        return super().create_report(
+            left=left,
+            right=right,
+            hat_switch=hat_switch,
+            buttons=buttons,
+            reportID=reportID,
+            application=application,
+        )
 
     def store_right_joystick(self, gamepad, data):
         gamepad.rudder, gamepad.throttle = data
 
 
 class SaitekGamepad(JoystickGamepad):
+    # fmt: off
     report_descriptor = [
         0x05, 0x01,                    # Usage Page (Generic Desktop)        0
         0x09, 0x04,                    # Usage (Joystick)                    2
@@ -505,13 +541,15 @@ class SaitekGamepad(JoystickGamepad):
         0xc0,                          # .End Collection                     623
         0xc0,                          # End Collection                      624
     ]
+    # fmt: on
 
     def __init__(self, rdesc=report_descriptor, name=None):
-        super().__init__(rdesc, name=name, input_info=(BusType.USB, 0x06a3, 0xff0d))
+        super().__init__(rdesc, name=name, input_info=(BusType.USB, 0x06A3, 0xFF0D))
         self.buttons = (1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12)
 
 
 class AsusGamepad(BaseGamepad):
+    # fmt: off
     report_descriptor = [
         0x05, 0x01,                    # Usage Page (Generic Desktop)        0
         0x09, 0x05,                    # Usage (Game Pad)                    2
@@ -601,7 +639,8 @@ class AsusGamepad(BaseGamepad):
         0xc0,                          # .End Collection                     191
         0xc0,                          # End Collection                      192
     ]
+    # fmt: on
 
     def __init__(self, rdesc=report_descriptor, name=None):
-        super().__init__(rdesc, name=name, input_info=(BusType.USB, 0x18d1, 0x2c40))
+        super().__init__(rdesc, name=name, input_info=(BusType.USB, 0x18D1, 0x2C40))
         self.buttons = (1, 2, 4, 5, 7, 8, 14, 15, 13)
